@@ -4,9 +4,10 @@
 
 #include <stdio.h>
 
-static const char *USERNAME = "libssh2"; /* configured in Dockerfile */
+/* configured in Dockerfile */
+static const char *USERNAME = "libssh2";
 static const char *KEY_FILE_PRIVATE = "key_dsa";
-static const char *KEY_FILE_PUBLIC = "key_dsa.pub"; /* configured in Dockerfile */
+static const char *KEY_FILE_PUBLIC = "key_dsa.pub";
 
 int test(LIBSSH2_SESSION *session)
 {
