@@ -4,9 +4,10 @@
 
 #include <stdio.h>
 
-const char *USERNAME = "libssh2"; /* configured in Dockerfile */
-const char *KEY_FILE_PRIVATE = "key_rsa";
-const char *KEY_FILE_PUBLIC = "key_rsa.pub"; /* configured in Dockerfile */
+/* configured in Dockerfile */
+static const char *USERNAME = "libssh2";
+static const char *KEY_FILE_PRIVATE = "key_rsa";
+static const char *KEY_FILE_PUBLIC = "key_rsa.pub";
 
 int test(LIBSSH2_SESSION *session)
 {
@@ -14,12 +15,12 @@ int test(LIBSSH2_SESSION *session)
 
     const char *userauth_list =
         libssh2_userauth_list(session, USERNAME, strlen(USERNAME));
-    if (userauth_list == NULL) {
+    if(userauth_list == NULL) {
         print_last_session_error("libssh2_userauth_list");
         return 1;
     }
 
-    if (strstr(userauth_list, "publickey") == NULL) {
+    if(strstr(userauth_list, "publickey") == NULL) {
         fprintf(stderr, "'publickey' was expected in userauth list: %s\n",
                 userauth_list);
         return 1;
@@ -28,7 +29,7 @@ int test(LIBSSH2_SESSION *session)
     rc = libssh2_userauth_publickey_fromfile_ex(
         session, USERNAME, strlen(USERNAME), KEY_FILE_PUBLIC, KEY_FILE_PRIVATE,
         NULL);
-    if (rc != 0) {
+    if(rc != 0) {
         print_last_session_error("libssh2_userauth_publickey_fromfile_ex");
         return 1;
     }
